from typing import Iterator, Any

from datapipeline.sources.models.loader import DataLoader


class {{CLASS_NAME}}(DataLoader):
    """Custom loader stub. Yield raw items to be parsed by a parser."""

    def __iter__(self) -> Iterator[Any]:
        # TODO: implement data loading
        yield from ()
