kind: composed
id: {{ stream_id }}  # format: domain.dataset.(variant)
# cadence: ${group_by}          # optional per-contract cadence
# partition_by: <field or [fields]>
inputs:
  - {{ inputs_list }}

mapper:
  entrypoint: {{ mapper_entrypoint }}
  args: { driver: {{ driver_key }} }
